/*
 * This file is part of DesktopBSD, see the README file.
 *
 * Author: Peter Hofer <hofer.p@gmail.com>
 *			(C) 2004, 2005
 *
 * Copyright: BSD, see the COPYING file included in this distribution
 *
 */

#include <qtimer.h>
#include <qprocess.h>
#include <qeventloop.h>
#include <qapplication.h>
#include <qprogressdialog.h>

#include <dbsdglobal.h>
#include <dialogs.h>
#include <fastestcvsupengine.h>
#include <unixprocess.h>
#include <pkgupgradeengine.h>

/* ATTENTION
 *	The IDs in bgrpPkgUse must correspond to PkgUpgradeEngine::PackageUse!
 */

void frmPkgSettings::fastestCVSUp() {
	editCVSUpServer->setText("");

	fcprocess = FastestCVSUpEngine::getProcess();
	if(fcprocess != NULL) {
		connect(fcprocess, SIGNAL(processExited()), SLOT(fastestCVSUpExited()));
	
		if(fcprocess->start()) {
			fcpdlg = new QProgressDialog(this, NULL, true);
			disconnect(fcpdlg, SIGNAL(canceled()), fcpdlg, SLOT(cancel()));
			connect(fcpdlg, SIGNAL(canceled()), SLOT(fastestCVSUpCanceled()));
			fcpdlg->setCaption("DesktopBSD");
			fcpdlg->setTotalSteps(0); /* busy indicator */
			
			fastestCVSUpProgress();
		} else
			Dialogs::warnDlg(tr("Error: fastest_cvsup couldn't be started."));
	}
}

void frmPkgSettings::accept() {
	settings.setPackageUse(bgrpPkgUse->selectedId());
	settings.setPortsUpdateMethod(bgrpPortsUpdateMethod->selectedId());
	settings.setRecursive(chkRecursive->isChecked());
	settings.setUpwardRecursive(chkUpwardRecursive->isChecked());
	settings.setKeepGoing(chkKeepGoing->isChecked());
	settings.setMakeArguments(QStringList::split(' ', editMakeArgs->text()));
	settings.setMakeEnvironment(QStringList::split(' ', editMakeEnv->text()));
	settings.setCvsupServer(editCVSUpServer->text());
	settings.setPackageDirectory(editPackageDir->text());

	QDialog::accept();
}


void frmPkgSettings::init() {
	bgrpPkgUse->setButton(settings.getPackageUse());
	chkRecursive->setChecked(settings.getRecursive());
	chkUpwardRecursive->setChecked(settings.getUpwardRecursive());
	chkKeepGoing->setChecked(settings.getKeepGoing());
	editMakeArgs->setText(settings.getMakeArguments().join(" "));
	editMakeEnv->setText(settings.getMakeEnvironment().join(" "));
	editCVSUpServer->setText(settings.getCvsupServer());
	editPackageDir->setText(settings.getPackageDirectory());
	lblDefaultPackageDir->setText(tr("Default: <i>%1</i>").arg(PkgUpgradeEngine::readDefaultRepository()));

	int updatemethod = settings.getPortsUpdateMethod();
	bgrpPortsUpdateMethod->setButton(updatemethod);
	grpCvsupSettings->setEnabled(updatemethod == 1);

	btnFastestCVSUp->setIconSet(DBSDGlobal::getIconLoader()->loadIcon("find", DBSDIconLoader::Button));
}


void frmPkgSettings::fastestCVSUpExited() {
	if(fcprocess->normalExit() && fcprocess->exitStatus() == 0)
		editCVSUpServer->setText(fcprocess->readLineStdout());
	
	delete fcprocess;
	fcprocess = NULL;
}


void frmPkgSettings::fastestCVSUpProgress() {
	if(fcprocess != NULL) {
		fcpdlg->setProgress(fcpdlg->progress() + 8);
		QTimer::singleShot(30, this, SLOT(fastestCVSUpProgress()));
	} else {
		delete fcpdlg;
		fcpdlg = NULL;
	}
}


void frmPkgSettings::fastestCVSUpCanceled() {
	if(fcprocess != NULL)
		fcprocess->tryTerminate();
}



void frmPkgSettings::openServerList(const QString & url) {
	UnixProcess process(QStringList::split(' ', "/usr/local/bin/kfmclient exec"));
	process.addArgument(url);

	if(!process.start())
		Dialogs::warnDlg(tr("Error: Cannot launch browser.\n"
								  "(kfmclient couldn't be started)"));
}


void frmPkgSettings::customMakeInfo() {
	if(bgrpPkgUse->selected() == radUsePackages && !editMakeArgs->text().isEmpty())
		Dialogs::infoDlg(tr("You have selected \"Binary packages whenever possible\" as installation\n"
								  "method, but since custom make arguments are specified, software will\n"
								  "be always built from source."));
}


void frmPkgSettings::updateMethodChanged(int index)
{
	grpCvsupSettings->setEnabled(index == 1);
}
